﻿#region

using System;
using System.Drawing;
using System.Windows.Forms;

#endregion

namespace LoadReview.Grid
{
    public class DataGridCustomColumnBase : DataGridTextBoxColumn
    {
        #region Delegates

        public delegate void DataGridTextBoxColumnEventHandler(object sender, DataGridTextBoxColumnEventArgs e);

        #endregion

        private Color alternatingBackColor = SystemColors.Window;
        private Color backColor = SystemColors.Window;

        private int columnNumber = -1;

        //controle utilizado para edição da coluna
        private Control control;

        //dimensões da célua
        private Rectangle dimensao = Rectangle.Empty;

        //cor da fonte da célula
        private Color foreColor = SystemColors.WindowText;
        private DataGrid owner;
        private bool readOnly;


        //numero da linha no Grid
        private int rowNumber;
        private StringFormat stringFormat;

        //texto contido na célula

        public DataGridCustomColumnBase()
        {
            rowNumber = 0;
            readOnly = false;
            control = null;
            owner = null;
            stringFormat = null;
            Text = "";
        }

        public string Text { get; set; }


        // Geters e Setters

        //cor  da fonte
        public Color ForeColor
        {
            get { return foreColor; }
            set { foreColor = value; }
        }

        //back color
        public Color BackColor
        {
            get { return backColor; }
            set { backColor = value; }
        }

        //Cor da linha alternada
        public Color AlternatingBackColor
        {
            get { return alternatingBackColor; }
            set
            {
                alternatingBackColor = value;
                Invalidate();
            }
        }

        //define alinhamento
        public virtual HorizontalAlignment Alignment
        {
            get
            {
                return (StringFormat.Alignment == StringAlignment.Center)
                           ?
                               HorizontalAlignment.Center
                           :
                               (StringFormat.Alignment == StringAlignment.Far)
                                   ?
                                       HorizontalAlignment.Right
                                   : HorizontalAlignment.Left;
            }

            set
            {
                StringFormat.Alignment = (value == HorizontalAlignment.Center)
                                             ?
                                                 StringAlignment.Center
                                             :
                                                 (value == HorizontalAlignment.Right)
                                                     ?
                                                         StringAlignment.Far
                                                     : StringAlignment.Near;

                Invalidate();
            }
        }


        //define se a coluna é somente leitura
        public bool ReadOnly
        {
            get { return readOnly; }
            set
            {
                readOnly = value;
                Invalidate();
            }
        }

        //define formatacao da célula
        public StringFormat StringFormat
        {
            get
            {
                if (stringFormat == null)
                {
                    stringFormat = new StringFormat();
                    Alignment = HorizontalAlignment.Left;
                }
                return stringFormat;
            }
        }


        // indica um valor para coluna quando a mesma for nula
        public object NullValue
        {
            get { return NullText; }
            set { NullText = value.ToString(); }
        }

        //linha atual da célula
        public int RowNumber
        {
            get { return rowNumber; }
        }

        //coluna atual da célula
        public int ColumnNumber
        {
            get
            {
                if ((columnNumber == -1) && (owner != null))
                {
                    foreach (DataGridTableStyle table in owner.TableStyles)
                    {
                        columnNumber = table.GridColumnStyles.IndexOf(this);
                        if (columnNumber != -1)
                        {
                            break;
                        }
                    }
                }
                return columnNumber;
            }
        }

        //retorna o DataGrid onde a coluna está vinculada
        public DataGrid Owner
        {
            get
            {
                if (owner == null)
                {
                    throw new InvalidOperationException("Coluna não está " + "vinculada a um DataGrid");
                }
                return owner;
            }
            set { owner = value; }
        }


        //responsável pelo vinculo da coluna com o controle(TextBox, checkBox etc)
        public Control HostedControl
        {
            get
            {
                if ((control == null) && (Owner != null))
                {
                    control = CreateHostedControl();
                    control.Visible = false;
                    control.Name = HeaderText;
                    control.Font = Owner.Font;
                    Owner.Controls.Add(control);

                    control.DataBindings.Add(GetBoundPropertyName(), Owner.DataSource, MappingName,
                                             true, DataSourceUpdateMode.OnValidation, NullValue);

                    // ReSharper disable RedundantAssignment
                    HScrollBar horisonal = null;
                    // ReSharper restore RedundantAssignment
                    foreach (Control c in Owner.Controls)
                    {
                        if ((horisonal = c as HScrollBar) != null)
                        {
                            // ReSharper disable RedundantDelegateCreation
                            horisonal.ValueChanged += new EventHandler(GridScrolled);
                            // ReSharper restore RedundantDelegateCreation
                            break;
                        }
                    }
                }
                return control;
            }
        }

        public event DataGridTextBoxColumnEventHandler OnDrawColumnCell;

        //


        //retorno nome da propriedade atualizável do controle. No caso do TextBox retorna Text.
        protected virtual string GetBoundPropertyName()
        {
            throw new InvalidOperationException(
                "Propriedade deve ser definida " + "sobrescrevendo GetBoundPropertyName()");
        }

        //cria o controle propriamente dito
        protected virtual Control CreateHostedControl()
        {
            throw new InvalidOperationException("Controle deve ser definido " +
                                                "sobrescrevendo CreateHostedControl()");
        }


        //Gerencia a atualização do controle. Verifica se célula foi clicada, coloca ou remove foco do controle, habilita para edição.
        protected virtual void UpdateHostedControl()
        {
            Rectangle selectedBounds = Owner.GetCellBounds(Owner.CurrentCell.RowNumber,
                                                           Owner.CurrentCell.ColumnNumber);
            if (!ReadOnly && (ColumnNumber ==
                              Owner.CurrentCell.ColumnNumber) &&
                Owner.HitTest(selectedBounds.Left, selectedBounds.Top).Type
                == DataGrid.HitTestType.Cell &&
                Owner.HitTest(selectedBounds.Right, selectedBounds.Bottom).Type ==
                DataGrid.HitTestType.Cell)
            {
                if (selectedBounds != dimensao)
                {
                    dimensao = selectedBounds;
                    HostedControl.Bounds = selectedBounds;
                    HostedControl.Focus();
                    HostedControl.Update();
                }

                if (!HostedControl.Visible)
                {
                    HostedControl.Show();
                    HostedControl.Focus();
                }
            }
            else if (HostedControl.Visible)
            {
                HostedControl.Hide();
            }
        }


        //desenha a célula 
        // ReSharper disable RedundantAssignment
        protected override void Paint(Graphics g, Rectangle bounds,
                                      CurrencyManager source, int rowNum,
                                      Brush backBrush, Brush foreBrush, bool alignToRight)

        {
            // ReSharper disable JoinDeclarationAndInitializer
            RectangleF textBounds;
            Object cellData;
            // ReSharper restore JoinDeclarationAndInitializer


            cellData = PropertyDescriptor.GetValue(source.List[rowNum]);

            rowNumber = rowNum;
            Text = cellData.ToString();

            if (OnDrawColumnCell != null)
            {
                OnDrawColumnCell(this, new DataGridTextBoxColumnEventArgs(this));
            }

            DrawBackground(g, bounds, rowNum, backBrush);
            bounds.Inflate(-2, -2);

            textBounds = new RectangleF(bounds.X, bounds.Y, bounds.Width, bounds.Height);
            foreBrush = new SolidBrush(foreColor);

            g.DrawString(FormatText(cellData), Owner.Font, foreBrush, textBounds, StringFormat);

            UpdateHostedControl();
        }

        //Desenha o fundo da célula
        protected virtual void DrawBackground(Graphics g, Rectangle bounds, int rowNum,
                                              Brush backbrush)
        {
            Brush background = backbrush;
            Color color = backColor;

            // ReSharper disable ConditionIsAlwaysTrueOrFalse
            if (alternatingBackColor != null)
                // ReSharper restore ConditionIsAlwaysTrueOrFalse
            {
                if ((background != null) && ((rowNum & 1) != 0) && !Owner.IsSelected(rowNum))
                {
                    color = alternatingBackColor;
                }
            }
            background = new SolidBrush(color);
            g.FillRectangle(background, bounds);
        }


        //formata e alinha o texto
        protected virtual String FormatText(Object cellData)
        {
            String cellText;

            if ((cellData == null) || (DBNull.Value == cellData))
            {
                cellText = NullText;
            }
            else if (cellData is IFormattable)
            {
                cellText = ((IFormattable) cellData).ToString(Format, FormatInfo);
            }
            else if (cellData is IConvertible)
            {
                cellText = ((IConvertible) cellData).ToString(FormatInfo);
            }
            else
            {
                cellText = cellData.ToString();
            }
            return cellText;
        }

        protected void Invalidate()
        {
            if (Owner != null)
            {
                Owner.Invalidate();
            }
        }

        private void GridScrolled(Object sender, EventArgs e)
        {
            UpdateHostedControl();
        }

        #region Nested Type: DataGridTextBoxColumnEventArgs

        public class DataGridTextBoxColumnEventArgs : EventArgs
        {
            public DataGridCustomColumnBase column;

            public DataGridTextBoxColumnEventArgs(
                DataGridCustomColumnBase column)
            {
                this.column = column;
            }
        }

        #endregion
    }
}